@@ -1,63 +1,125 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
-__author__ = 'Jathan McCollum'
-__email__ = 'jathan@gmail.com'
-__version__ = '0.6.4'
 
 """
-Intelligently parse IPv4/IPv6 addresses, CIDRs, ranges, and wildcard matches to
-attempt return a valid list of IP addresses. It's smart enough to fix bad
-network boundaries for you.
+IP address parsing for humans.
 
-The cidrize() function is the public interface. The module may also be run
+Cidrize takes IP address inputs that people tend to use in practice, validates
+them, and converts them to objects.
+
+It will intelligently parse IPv4/IPv6 addresses, CIDRs, ranges, and wildcard
+matches to attempt return a valid list of IP addresses. It's smart enough to fix
+bad network boundaries for you.
+
+The ``cidrize()`` function is the public interface. The module may also be run
 interactively for debugging purposes.
 """
 
 import itertools
 import logging
-from netaddr import (AddrFormatError, IPAddress, IPGlob, IPNetwork, IPRange,
-                     IPSet, spanning_cidr)
+from optparse import OptionParser  # pylint: disable=deprecated-module
 import os
 import re
 import socket
 import sys
 
+from netaddr import (
+    AddrFormatError,
+    IPAddress,
+    IPGlob,
+    IPNetwork,
+    IPRange,
+    IPSet,
+    spanning_cidr,
+)
+
+
 # Globals
-EVERYTHING = ['internet at large', '*', 'all', 'any', 'internet', '0.0.0.0',
-              '0.0.0.0/0', '0.0.0.0-255.255.255.255']
-MAX_RANGE_LEN = 16384 # This is a /18
+EVERYTHING = [
+    "internet at large",
+    "*",
+    "all",
+    "any",
+    "internet",
+    "0.0.0.0",
+    "0.0.0.0/0",
+    "0.0.0.0-255.255.255.255",
+]
+# IPRange objects larger than MAX_RANGE_LEN will always be strict.
+# This is an IPv4 /18
+MAX_RANGE_LEN = 16384
 
 # Setup logging
-DEBUG = os.getenv('DEBUG', False)
+DEBUG = os.getenv("DEBUG")
 LOG_LEVEL = logging.INFO
 LOG_FORMAT = "%(asctime)s [%(levelname)s]: %(message)s"
 if DEBUG:
     LOG_LEVEL = logging.DEBUG
 
 logging.basicConfig(level=LOG_LEVEL, format=LOG_FORMAT)
-log = logging
+log = logging.getLogger(__name__)
 
+#
 # Pre-compiled re patterns. You know, for speed!
-cidr_re = re.compile(r"\d+\.\d+\.\d+\.\d+(?:\/\d+)?$")
-range_re = re.compile(r"\d+\.\d+\.\d+\.\d+\-\d+\.\d+\.\d+\.\d+$")
-glob_re = re.compile(r"\d+\.\d+\.\d+\.\*$")
-bracket_re = re.compile(r"(.*?)\.(\d+)[\[\{\(](.*)[\)\}\]]$") # parses '1.2.3.4[5-9]' or '1.2.3.[57]'
-hyphen_re = re.compile(r"(.*?)\.(\d+)\-(\d+)$")
-hostname_re = re.compile(r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?)', re.IGNORECASE)
+#
+
+# Old-fashioned CIDR notation (192.168.1.0/24)
+RE_CIDR = re.compile(r"\d+\.\d+\.\d+\.\d+(?:\/\d+)?$")
+
+# 1.2.3.118-1.2.3.121 range style
+RE_RANGE = re.compile(r"\d+\.\d+\.\d+\.\d+\-\d+\.\d+\.\d+\.\d+$")
+
+# 2001::14-2002::1.2.3.121 range style
+RE_RANGE6 = re.compile(
+    r"[0-9a-fA-F]+:[0-9A-Fa-f:.]+\-[0-9a-fA-F]+:[0-9A-Fa-f:.]+$"
+)
+
+# 1.2.3.* glob style
+RE_GLOB = re.compile(r"\d+\.\d+\.\d+\.\*$")
+
+# 1.2.3.4[5-9] or 1.2.3.[49] bracket style as a last resort
+RE_BRACKET = re.compile(r"(.*?)\.(\d+)[\[\{\(](.*)[\)\}\]]$")
+
+# 1.2.3.4-70 hyphen style
+RE_HYPHEN = re.compile(r"(.*?)\.(\d+)\-(\d+)$")
+
+# Hostnames (we're assuming first char is alpha)
+RE_HOSTNAME = re.compile(
+    r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?)",
+    re.IGNORECASE,
+)
 
 
 # Exports
-__all__ = ('cidrize', 'CidrizeError', 'dump', 'normalize_address',
-           'optimize_network_usage', 'parse_range', 'is_ipv6')
+__all__ = (
+    "cidrize",
+    "CidrizeError",
+    "dump",
+    "normalize_address",
+    "optimize_network_range",
+    "parse_range",
+    "is_ipv6",
+)
 
 
 # Exceptions
-class CidrizeError(AddrFormatError): pass
-class NotCIDRStyleError(CidrizeError): pass
-class NotRangeStyleError(CidrizeError): pass
-class NotGlobStyleError(CidrizeError): pass
-class NotBracketStyleError(CidrizeError): pass
+class CidrizeError(AddrFormatError):
+    """Generic Cidrize error."""
+
+
+class NotCIDRStyle(CidrizeError):
+    """Not CIDR style?"""
+
+
+class NotRangeStyle(CidrizeError):
+    """Not range style?"""
+
+
+class NotGlobStyle(CidrizeError):
+    """Not glob style?"""
+
+
+class NotBracketStyle(CidrizeError):
+    """Not bracket style?"""
 
 
 # Functions
@@ -74,28 +136,31 @@ def parse_brackets(text):
     :param text:
         The string to parse.
     """
-    match = bracket_re.match(text)
+
+    match = RE_BRACKET.match(text)
+
     if match is None:
         return None
 
     parts = match.groups()
+
     # '1.2.3.4[5-9] style
-    log.debug('parse_brackets() parts: %r' % (parts,))
+    log.debug("parse_brackets() parts: %r", (parts,))
     if len(parts) == 3:
         prefix, subnet, enders = parts
-        network = '.'.join((prefix, subnet))
+        network = ".".join((prefix, subnet))
 
     # '1.2.3.[5-9] style
     elif len(parts) == 2:
         prefix, enders = parts
-        network = prefix + '.'
+        network = prefix + "."
 
     else:
-        raise NotBracketStyleError("Bracketed style not parseable: '%s'" % text)
+        raise NotBracketStyle("Bracketed style not parseable: '%s'" % text)
 
     # Split hyphenated [x-y]
-    if '-' in enders:
-        first, last = enders.split('-')
+    if "-" in enders:
+        first, last = enders.split("-")
 
     # Get first/last from [xy] - This really only works with single
     # digits
@@ -108,6 +173,7 @@ def parse_brackets(text):
 
     return IPRange(network + first, network + last)
 
+
 def parse_hyphen(text):
     """
     Parses a hyphen in the last octet, e.g. '1.2.3.4-70'
@@ -115,16 +181,17 @@ def parse_hyphen(text):
     :param text:
         The string to parse
     """
-    match = hyphen_re.match(text)
+    match = RE_HYPHEN.match(text)
     if match is None:
         return None
 
     parts = match.groups()
     prefix, start, finish = parts
-    network = prefix + '.'
+    network = prefix + "."
 
     return IPRange(network + start, network + finish)
 
+
 def parse_range(ipstr):
     """
     Given a hyphenated range of IPs, return an IPRange object.
@@ -132,13 +199,14 @@ def parse_range(ipstr):
     :param ipstr:
         The hyphenated IP range.
     """
-    start, finish = ipstr.split('-')
-    ip = IPRange(start, finish)
-    log.debug('parse_range_style() start: %r' % start)
-    log.debug('parse_range_style() finish: %r' % finish)
-    log.debug('parse_range_style() ip: %r' % ip)
+    start, finish = ipstr.split("-")
+    ip_range = IPRange(start, finish)
+    log.debug("parse_range_style() start: %r", start)
+    log.debug("parse_range_style() finish: %r", finish)
+    log.debug("parse_range_style() ip: %r", ip_range)
+
+    return ip_range
 
-    return ip
 
 def parse_commas(ipstr, **kwargs):
     """
@@ -153,8 +221,8 @@ def parse_commas(ipstr, **kwargs):
         A comma-separated string of IP address patterns.
     """
     # Clean whitespace before we process
-    ipstr = ipstr.replace(' ', '').strip()
-    items = ipstr.split(',')
+    ipstr = ipstr.replace(" ", "").strip()
+    items = ipstr.split(",")
 
     # Possibly nested depending on input, so we'll run it thru itertools.chain
     # to flatten it. Then we make it a IPSet to optimize adjacencies and finally
@@ -165,6 +233,7 @@ def parse_commas(ipstr, **kwargs):
 
     return ipset.iter_cidrs()
 
+
 def is_ipv6(ipstr):
     """
     Checks whether a string is IPv6 or not. Doesn't handle addresses with
@@ -176,14 +245,15 @@ def is_ipv6(ipstr):
     :param ipstr:
         A suspected IPv6 address
     """
-    log.debug('is_ipv6() got: %r' % ipstr)
+    log.debug("is_ipv6() got: %r", ipstr)
     try:
         socket.inet_pton(socket.AF_INET6, ipstr)
         return True
     except (socket.error, AttributeError):
         return False
 
-def cidrize(ipstr, strict=False, modular=True):
+
+def cidrize(ipstr, strict=False, raise_errors=True):
     """
     This function tries to determine the best way to parse IP addresses correctly & has
     all the logic for trying to do the right thing!
@@ -214,18 +284,19 @@ def cidrize(ipstr, strict=False, modular=True):
 
     Defaults:
 
-        * parsing exceptions will raise a CidrizeError (modular=True).
+        * parsing exceptions will raise a CidrizeError (raise_errors=True).
         * results will be returned as a spanning CIDR (strict=False).
 
     :param ipstr:
         IP string to be parsed.
 
-    :param modular:
+    :param raise_errors:
         Set to False to cause exceptions to be stripped & the error text will be
         returned as a list. This is intended for use with scripts or APIs
         out-of-the box.
 
-    Example:
+    Example::
+
         >>> import cidrize as c
         >>> c.cidrize('1.2.3.4-1.2.3.1099')
         Traceback (most recent call last):
@@ -234,13 +305,14 @@ def cidrize(ipstr, strict=False, modular=True):
             raise CidrizeError(err)
         cidrize.CidrizeError: base address '1.2.3.1099' is not IPv4
 
-        >>> c.cidrize('1.2.3.4-1.2.3.1099', modular=False)
+        >>> c.cidrize('1.2.3.4-1.2.3.1099', raise_errors=False)
         ["base address '1.2.3.1099' is not IPv4"]
 
     :param strict:
         Set to True to return explicit networks based on start/end addresses.
 
-    Example:
+    Example::
+
         >>> import cidrize as c
         >>> c.cidrize('1.2.3.4-1.2.3.10')
         [IPNetwork('1.2.3.0/28')]
@@ -249,15 +321,15 @@ def cidrize(ipstr, strict=False, modular=True):
         [IPNetwork('1.2.3.4/30'), IPNetwork('1.2.3.8/31'), IPNetwork('1.2.3.10/32')]
 
     """
-    ip = None
+    ipobj = None
 
     # Short-circuit to parse commas since it calls back here anyway
-    if ',' in ipstr:
-        return parse_commas(ipstr, strict=strict, modular=modular)
+    if "," in ipstr:
+        return parse_commas(ipstr, strict=strict, raise_errors=raise_errors)
 
     # Short-circuit for hostnames (we're assuming first char is alpha)
-    if hostname_re.match(ipstr):
-        raise CidrizeError('Cannot parse hostnames!')
+    if RE_HOSTNAME.match(ipstr):
+        raise CidrizeError("Cannot parse hostnames!")
 
     # Otherwise try everything else
     result = None
@@ -265,44 +337,51 @@ def cidrize(ipstr, strict=False, modular=True):
         # Parse "everything" & immediately return; strict/loose doesn't apply
         if ipstr in EVERYTHING:
             log.debug("Trying everything style...")
-            return [IPNetwork('0.0.0.0/0')]
+            return [IPNetwork("0.0.0.0/0")]
 
         # Parse old-fashioned CIDR notation & immediately return; strict/loose doesn't apply
         # Now with IPv6!
-        elif cidr_re.match(ipstr) or is_ipv6(ipstr):
+        elif RE_CIDR.match(ipstr) or is_ipv6(ipstr):
             log.debug("Trying CIDR style...")
-            ip = IPNetwork(ipstr)
-            return [ip.cidr]
+            ipobj = IPNetwork(ipstr)
+            return [ipobj.cidr]
 
         # Parse 1.2.3.118-1.2.3.121 range style
-        elif range_re.match(ipstr):
+        elif RE_RANGE.match(ipstr):
             log.debug("Trying range style...")
             result = parse_range(ipstr)
 
+        # Parse 2001::14-2002::1.2.3.121 range style
+        elif RE_RANGE6.match(ipstr):
+            log.debug("Trying range6 style...")
+            result = parse_range(ipstr)
+
         # Parse 1.2.3.4-70 hyphen style
-        elif hyphen_re.match(ipstr):
+        elif RE_HYPHEN.match(ipstr):
             log.debug("Trying hyphen style...")
             result = parse_hyphen(ipstr)
 
         # Parse 1.2.3.* glob style
-        elif glob_re.match(ipstr):
+        elif RE_GLOB.match(ipstr):
             log.debug("Trying glob style...")
             ipglob = IPGlob(ipstr)
             result = spanning_cidr(ipglob)
 
         # Parse 1.2.3.4[5-9] or 1.2.3.[49] bracket style as a last resort
-        elif bracket_re.match(ipstr):
+        elif RE_BRACKET.match(ipstr):
             log.debug("Trying bracket style...")
             result = parse_brackets(ipstr)
 
         # If result still isn't set, let's see if it's IPv6??
         elif result is None:
             log.debug("Trying bare IPv6 parse...")
-            result  = IPNetwork(ipstr)
+            result = IPNetwork(ipstr)
 
         # This will probably fail 100% of the time. By design.
         else:
-            raise CidrizeError("Could not determine parse style for '%s'" % ipstr)
+            raise CidrizeError(
+                "Could not determine parse style for '%s'" % ipstr
+            )
 
         # If it's a single host, just return it wrapped in a list
         if result.size == 1:
@@ -314,20 +393,25 @@ def cidrize(ipstr, strict=False, modular=True):
         # IPRange objects larger than MAX_RANGE_LEN will always be strict.
         if not strict:
             if isinstance(result, IPRange) and result.size >= MAX_RANGE_LEN:
-                log.debug('IPRange objects larger than /18 will always be strict.')
+                log.debug(
+                    "IPRange objects larger than /18 will always be strict."
+                )
                 return result.cidrs()
+            if isinstance(result, IPNetwork):
+                return [result.cidr]
             return [spanning_cidr(result)]
         else:
             try:
-                return result.cidrs() # IPGlob and IPRange have .cidrs()
+                return result.cidrs()  # IPGlob and IPRange have .cidrs()
             except AttributeError as err:
-                return result.cidr    # IPNetwork has .cidr
+                return [result.cidr]  # IPNetwork has .cidr
 
     except (AddrFormatError, TypeError, ValueError) as err:
-        if modular:
-            raise CidrizeError(err)
+        if raise_errors:
+            raise CidrizeError(str(err)) from err
         return [str(err)]
 
+
 def optimize_network_range(ipstr, threshold=0.9, verbose=DEBUG):
     """
     Parses the input string and then calculates the subnet usage percentage. If over
@@ -358,7 +442,7 @@ def optimize_network_range(ipstr, threshold=0.9, verbose=DEBUG):
 
     """
     if threshold > 1 or threshold < 0:
-        raise CidrizeError('Threshold must be from 0.0 to 1.0')
+        raise CidrizeError("Threshold must be from 0.0 to 1.0")
 
     # Can't optimize 0.0.0.0/0!
     if ipstr in EVERYTHING:
@@ -369,20 +453,21 @@ def optimize_network_range(ipstr, threshold=0.9, verbose=DEBUG):
     ratio = float(len(strict)) / float(len(loose))
 
     if verbose:
-        print 'Subnet usage ratio: %s; Threshold: %s' % (ratio, threshold)
+        print("Subnet usage ratio: %s; Threshold: %s" % (ratio, threshold))
 
     if ratio >= threshold:
         if verbose:
-            print 'Over threshold, IP Parse Mode: LOOSE'
+            print("Over threshold, IP Parse Mode: LOOSE")
         result = loose.iter_cidrs()
     else:
         if verbose:
-            print 'Under threshold, IP Parse Mode: STRICT'
+            print("Under threshold, IP Parse Mode: STRICT")
         result = strict.iter_cidrs()
 
     return result
 
-def output_str(ipobj, sep=', '):
+
+def output_str(ipobj, sep=", "):
     """
     Returns a character-separated string of constituent CIDR blocks for a given
     IP object (should support both IPy and netaddr objects).
@@ -395,6 +480,7 @@ def output_str(ipobj, sep=', '):
     """
     return sep.join([str(x) for x in ipobj])
 
+
 def normalize_address(ipstr):
     """
     Attempts to cleanup an IP address that is in a non-standard format such
@@ -404,8 +490,8 @@ def normalize_address(ipstr):
     :param ipstr:
         An IP address string.
     """
-    data = ipstr.split('/')
-    cidr = '32'
+    data = ipstr.split("/")
+    cidr = "32"
     if len(data) == 1:
         myip = data[0]
     elif len(data) == 2:
@@ -413,9 +499,10 @@ def normalize_address(ipstr):
     else:
         return ipstr
 
-    octets = (int(i) for i in myip.split('.'))
-    ip = '.'.join([str(o) for o in octets])
-    return '{0}/{1}'.format(ip, cidr)
+    octets = (int(i) for i in myip.split("."))
+    ipobj = ".".join([str(o) for o in octets])
+    return "{0}/{1}".format(ipobj, cidr)
+
 
 def netaddr_to_ipy(iplist):
     """
@@ -427,7 +514,7 @@ def netaddr_to_ipy(iplist):
         A list of netaddr.IPNetwork objects.
     """
     try:
-        import IPy
+        import IPy  # pylint: disable=import-outside-toplevel
     except ImportError:
         return iplist
 
@@ -436,6 +523,7 @@ def netaddr_to_ipy(iplist):
 
     return [IPy.IP(str(x)) for x in iplist]
 
+
 def dump(cidr):
     """
     Dumps a lot of info about a CIDR.
@@ -450,22 +538,22 @@ def dump(cidr):
         cidr = spanning_cidr(cidr)
 
     # Is this a /32 or /128?
-    single = (cidr.size == 1)
+    single = cidr.size == 1
 
-    log.debug('dump(): Single? %r' % single)
-    log.debug('dump(): Got? %r' % cidr)
+    log.debug("dump(): Single? %r", single)
+    log.debug("dump(): Got? %r", cidr)
 
     ip_first = IPAddress(cidr.first)
-    ip_firsthost = ip_first if single else cidr.iter_hosts().next()
+    ip_firsthost = ip_first if single else next(cidr.iter_hosts())
     ip_gateway = IPAddress(cidr.last - 1)
     ip_bcast = cidr.broadcast
     ip_netmask = cidr.netmask
     ip_hostmask = cidr.hostmask
     num_hosts = 1 if single else (cidr.last - 1) - cidr.first
 
-    out  = ''
+    out = ""
     out += "Information for %s\n\n" % cidr
-    out += "IP Version:\t\t%s\n" %  cidr.version
+    out += "IP Version:\t\t%s\n" % cidr.version
     out += "Spanning CIDR:\t\t%s\n" % cidr
     out += "Block Start/Network:\t%s\n" % ip_first
     out += "1st host:\t\t%s\n" % ip_firsthost
@@ -478,16 +566,25 @@ def dump(cidr):
 
     return out
 
+
 def parse_args(argv):
     """Parses args."""
-    from optparse import OptionParser
-    parser = OptionParser(usage='%prog [-v] [-d] [ip network]', add_help_option=0, description='''\
-Cidrize parses IP address notation and returns valid CIDR blocks. If you want
-debug output set the DEBUG environment variable.''')
 
-    parser.add_option('-h', '--help', action="store_false")
-    parser.add_option('-v', '--verbose', action='store_true',
-        help='Be verbose with user-friendly output. Lots of detail.')
+    parser = OptionParser(
+        usage="%prog [-v] [-d] [ip network]",
+        add_help_option=0,
+        description="""\
+Cidrize parses IP address notation and returns valid CIDR blocks. If you want
+debug output set the DEBUG environment variable.""",
+    )
+
+    parser.add_option("-h", "--help", action="store_false")
+    parser.add_option(
+        "-v",
+        "--verbose",
+        action="store_true",
+        help="Be verbose with user-friendly output. Lots of detail.",
+    )
 
     notes = """
     Intelligently take IPv4 addresses, CIDRs, ranges, and wildcard matches to attempt
@@ -521,40 +618,44 @@ debug output set the DEBUG environment variable.''')
     def phelp():
         """I help."""
         parser.print_help()
-        print notes
+        print(notes)
 
     if opts.help or len(args) == 1:
         phelp()
-        sys.exit('ERROR: You must specify an ip address. See usage information above!!')
+        sys.exit(
+            "ERROR: You must specify an ip address. See usage information above!!"
+        )
     else:
         opts.ip = args[1]
 
-    if ',' in opts.ip:
+    if "," in opts.ip:
         phelp()
         sys.exit("ERROR: Comma-separated arguments aren't supported!")
 
     return opts, args
 
+
 def main():
     """
     Used by the 'cidr' command that is bundled with the package.
     """
     opts, args = parse_args(sys.argv)
 
-    log.debug('OPTS: %r' % opts)
-    log.debug('ARGS: %r' % args)
+    log.debug("OPTS: %r", opts)
+    log.debug("ARGS: %r", args)
 
     ipstr = opts.ip
 
     try:
-        cidr = cidrize(ipstr, modular=False)
+        cidr = cidrize(ipstr, raise_errors=False)
         if cidr:
             if opts.verbose:
-                print dump(cidr),
+                print(dump(cidr))
             else:
-                print output_str(cidr)
+                print(output_str(cidr))
     except IndexError:
         return -1
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     sys.exit(main())
